# -*- coding: UTF-8 -*-
#
# Получение сообщений из очереди
#

import base64
import httplib2
import json
import logger
import traceback
from plugins import status
requestTimeout = 60
jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json"}
loggerSync = logger.getLogger()
messageTypeKey = 'type'
pluginPath = 'plugins'
goodMessageFile = 'lastGoodMessage.txt'


class Message(object):
    def __init__(self, id, content, properties={}):
        self.id = str(id)
        # бинарные данные при передаче через json закодированы с помощью base64
        self.content = base64.b64decode(content)
        self.properties = properties


def saveLastGoodMessageId(messageId):
    try:
        with open(goodMessageFile, 'w') as outfile:
            outfile.write(messageId)
    except Exception as e:
        loggerSync.warn(e)


def getLastGoodMessageId():
    try:
        with open(goodMessageFile, 'r') as infile:
            for line in infile:
                if line and line.strip():
                    return line
    except IOError as e:
        loggerSync.warn(e)
    return None


def getMessage(host, port, endpointId):
    loggerSync.info('Проверяем наличие сообщений в очереди')
    url = 'http://%s:%s/CSrest/rest/v2/queue/findFirst/%s' % (host, port, endpointId)
    http = httplib2.Http(timeout=requestTimeout)
    response, content = http.request(url, 'GET', headers=jsonHeader)
    if response and response.status == 200 and content:
        m = json.loads(content)
        return Message(m.get('id'), m.get('body'), m.get('messageProperties', {}))
    return None


def deleteMessage(host, port, endpointId, messageId):
    loggerSync.info('Удаляем сообщение из очереди')
    url = 'http://%s:%s/CSrest/rest/v2/queue/delete/%s?messageId=%s' % (host, port, endpointId, messageId)
    http = httplib2.Http(timeout=requestTimeout)
    response, content = http.request(url, 'DELETE', headers=jsonHeader)
    if not response or response.status != 200:
        raise Exception("Can't delete message %s from CS" % messageId)


def main(propertyReader):
    res = status.OK  # результат. Если проверка очереди прошла нормально, то вернется 1
    mess = 'Проверка сообщений закончена'
    try:
        message = getMessage(propertyReader.httpHostCS, propertyReader.httpPortCS, propertyReader.cashCode)
        while message:
            messageId = message.id
            if messageId and messageId == getLastGoodMessageId():
                loggerSync.info('Сообщение с идентификатором %s уже было обработано. Удаляем его.' % messageId)
                deleteMessage(propertyReader.httpHostCS, propertyReader.httpPortCS, propertyReader.cashCode, messageId)
                continue
            messageType = message.properties.get(messageTypeKey, '').lower()
            loggerSync.info('Получили сообщение')
            try:
                if messageType:
                    plugin = __import__('%s.%s' % (pluginPath, messageType), fromlist='q')
                    # выполняем скрипт, соответствующий типу сообщения
                    loggerSync.info('Вызываем плагин %s для обработки сообщения' % messageType.encode('utf8'))
                    try:
                        res, mess = plugin.main(message)
                    except Exception as e:
                        mess = str(e)
                        res = status.errorProgramm
                    if res == status.OK:
                        loggerSync.info('Удаляем из очереди принятое сообщение %s' % messageId)
                        saveLastGoodMessageId(messageId)
                        deleteMessage(propertyReader.httpHostCS, propertyReader.httpPortCS, propertyReader.cashCode, messageId)
                    else:
                        return res, mess
                else:
                    loggerSync.error("Тип сообщения не указан")
            except ImportError as e:
                res = status.errorProgramm
                mess = "Неизвестный тип сообщения. " + str(e)
                loggerSync.error(mess)
            loggerSync.info('')
            
            message = getMessage(propertyReader.httpHostCS, propertyReader.httpPortCS, propertyReader.cashCode)
            
    except Exception as e:
        res = status.errorProgramm
        mess = str(e)
        loggerSync.error(str(e))
        loggerSync.error(traceback.format_exc())
    finally:
        loggerSync.info('-----------------------')
    return res, mess


if __name__ == '__main__':
    import propertyReader
    main(propertyReader)
